//
//   This file contains the C++ code from Program 5.9 of
//   "Data Structures and Algorithms
//    with Object-Oriented Design Patterns in C++"
//   by Bruno R. Preiss.
//
//   Copyright (c) 1998 by Bruno R. Preiss, P.Eng.  All rights reserved.
//
//   http://www.pads.uwaterloo.ca/Bruno.Preiss/books/opus4/programs/pgm05_09.cpp
//

#ifndef __CONTAINER_H_
#define __CONTAINER_H_

#include "header.h"
//#include <typeinfo>

#include "Ownership.h"
#include "Hash.h"
#include "Iterator.h"
#include "Visitor.h"

class Container : public virtual Object , public virtual Ownership
{
protected:
    unsigned int count;

    Container ();
public:
    virtual unsigned int Count () const;
    virtual bool IsEmpty () const;
    virtual bool IsFull () const;
    virtual HashValue Hash () const;
    virtual void Put (std::ostream&) const;
    virtual Iterator& NewIterator () const;

    virtual void Purge () = 0;
    virtual void Accept (Visitor&) const = 0;
};

class PuttingVisitor : public Visitor
{
    std::ostream& stream;
    bool comma;
public:
    PuttingVisitor (std::ostream& s) : stream (s), comma (false)
	{}
    void Visit (Object& object)
    {
	if (comma)
	    stream << ", ";
//	stream << object;
	comma = true;
    }
};

class HashingVisitor : public Visitor
{
    HashValue value;
public:
    HashingVisitor (HashValue _value) : value (_value)
	{}
    void Visit (Object& object)
	{ value += object.Hash (); }
    HashValue Value () const
	{ return value; }
};

#endif
